"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tolatex = exports.replace_command_spacers = exports.load = exports.diacritics = exports.latex2unicode = exports.minimal = exports.bibtex = exports.biblatex = void 0;
exports.biblatex = require('./tables/biblatex.json');
exports.bibtex = require('./tables/bibtex.json');
exports.minimal = require('./tables/minimal.json');
const maps = { biblatex: exports.biblatex, bibtex: exports.bibtex, minimal: exports.minimal };
exports.latex2unicode = require('./tables/latex2unicode.json');
exports.diacritics = require('./tables/diacritics.json');
function load(mode, options) {
    let map = Object.assign({}, maps[mode].base);
    const packages = maps[mode].package;
    for (const pkg of (options.packages || []).map(p => packages[p]).filter(p => p)) {
        map = Object.assign(Object.assign({}, map), pkg);
    }
    for (const c of (options.text || '')) {
        if (map[c].text)
            delete map[c].math;
    }
    for (const c of (options.math || '')) {
        if (map[c].math)
            delete map[c].text;
    }
    for (const c of (options.ascii || '')) {
        if (exports.bibtex.base[c])
            map[c] = exports.bibtex.base[c];
    }
    if (options.charmap)
        map = Object.assign(Object.assign({}, map), options.charmap);
    return map;
}
exports.load = load;
function replace_command_spacers(latex) {
    return latex.replace(/\0(\s)/g, '{}$1').replace(/\0([^;.,!?${}_^\\/])/g, ' $1').replace(/\0/g, '');
}
exports.replace_command_spacers = replace_command_spacers;
const switchMode = {
    math: 'text',
    text: 'math',
};
const re = /([^\u0300-\u036F][\u0300-\u036F]+)|(i\uFE20a\uFE21)|([\uD800-\uDBFF][\uDC00-\uDFFF])|(.)/g;
function tolatex(text, table, options = {}) {
    options = Object.assign({ bracemath: true, mode: 'biblatex', preservecommandspacers: false, packages: new Set }, options);
    let mode = 'text';
    let braced = 0;
    const switchTo = {
        math: (options.bracemath ? '{$' : '$'),
        text: (options.bracemath ? '$}' : '$'),
    };
    let mapped;
    let switched;
    let m;
    let cd;
    let latex = '';
    text.replace(re, (match, cdpair, tie, pair, single) => {
        mapped = null;
        if (tie && !table[tie]) {
            mapped = { text: 'ia' };
        }
        else {
            mapped = table[tie] || table[cdpair] || table[pair] || table[single] || (cdpair && table[cdpair.normalize('NFC')]);
        }
        if (!mapped && options.mode !== 'minimal' && cdpair && (cd = exports.diacritics.tolatex[cdpair.substr(1)])) {
            let char;
            if (char = (table[cdpair[0]] || { text: cdpair[0], math: cdpair[0] })[cd.mode]) {
                const cmd = cd.command.match(/[a-z]/);
                if (options.mode === 'bibtex' && cd.mode === 'text') {
                    mapped = ({ [cd.mode]: `{\\${cd.command}${cmd ? ' ' : ''}${char}}` });
                }
                else if (cmd && char.length === 1) {
                    mapped = ({ [cd.mode]: `\\${cd.command} ${char}` });
                }
                else if (cmd) {
                    mapped = ({ [cd.mode]: `\\${cd.command}{${char}}` });
                }
                else {
                    mapped = ({ [cd.mode]: `\\${cd.command}${char}` });
                }
            }
        }
        if (!mapped)
            mapped = { text: match };
        if (!mapped[mode]) {
            mode = switchMode[mode];
            latex += switchTo[mode];
            switched = true;
        }
        else {
            switched = false;
        }
        switch (mapped[mode]) {
            case '\\{':
                braced += 1;
                break;
            case '\\}':
                braced -= 1;
                break;
        }
        if (braced < 0) {
            latex += '\\vphantom\\{';
            braced = 0;
        }
        if (switched && mode === 'text' && (m = latex.match(/([\^_])\{(.)\}(\$\}?)$/))) {
            latex = latex.slice(0, latex.length - m[0].length) + m[1] + m[2] + m[3];
        }
        latex += mapped[mode];
        if (mapped.commandspacer)
            latex += '\0';
        if (!switched && mode === 'math' && (m = latex.match(/(([\^_])\{[^{}]+)\}\2{(.\})$/))) {
            latex = latex.slice(0, latex.length - m[0].length) + m[1] + m[3];
        }
        if (mapped.alt) {
            for (const pkg of mapped.alt) {
                options.packages.add(pkg);
            }
        }
        return match;
    });
    switch (braced) {
        case 0:
            break;
        case 1:
            latex += '\\vphantom\\}';
            break;
        default:
            latex += `\\vphantom{${'\\}'.repeat(braced)}}`;
            break;
    }
    if (mode === 'math')
        latex += switchTo.text;
    if (!options.preservecommandspacers)
        latex = replace_command_spacers(latex);
    return latex;
}
exports.tolatex = tolatex;
